@@ -1,6 +1,6 @@
 from .libdyn import *
 from .BlockPrototypes import *
-from .TraverseGraph import *
+from .TraverseSignalFlowGraph import *
 from .Signal import *
 from .ExecutionCommands import *
 from .SystemManifest import *
@@ -182,27 +182,14 @@ def compile_single_system(system, reduce_uneeded_code = False):
     # call E.getExecutionLine( ) on them. Stop until no further dependend signal appear.
     # finally concatenare the execution lines
 
-    # 
-
     # start with following signals to be computed
-    #dependencySignals = executionLineToCalculateOutputs.dependencySignals
-    dependencySignalsSimulationInputs = executionLineToCalculateOutputs.dependencySignalsSimulationInputs
+    simulationInputSignalsToCalculateOutputs = executionLineToCalculateOutputs.dependencySignalsSimulationInputs
     blocksToUpdateStates = executionLineToCalculateOutputs.blocksToUpdateStates
     dependencySignalsThroughStates = executionLineToCalculateOutputs.dependencySignalsThroughStates
 
-    # get the simulation-input signals in dependencySignals TODO: collapse this
-    simulationInputSignalsToCalculateOutputs = []
-    for s in dependencySignalsSimulationInputs:
-
-        # # if isinstance(s, SimulationInputSignal):
-        # if s.is_crossing_system_boundary(system):
-            
-        simulationInputSignalsToCalculateOutputs.append(s)
-
     # counter for the order (i.e. step through all delays present in the system)
     order = 0
 
-
     # execution line per order
     commandToCalcTheResultsToPublish = CommandCalculateOutputs(system, 
                                                                 executionLineToCalculateOutputs, 
@@ -252,7 +239,7 @@ def compile_single_system(system, reduce_uneeded_code = False):
     blocksWhoseStatesToUpdate_All = []
 
     # find out which singnals must be further computed to allow a state-update of the blocks
-    dependencySignals__ = dependencySignalsThroughStates + dependencySignalsSimulationInputs
+    dependencySignals__ = dependencySignalsThroughStates + simulationInputSignalsToCalculateOutputs
 
     dependency_signals_through_states_that_are_already_computed = set()
     
@@ -298,8 +285,10 @@ def compile_single_system(system, reduce_uneeded_code = False):
         # list of commands for state update: 'commandsToExecuteForStateUpdate'
         
         #
-        # TODO: ensure somehow that variables are reserved for the inputs to the blocks
-        #       whose states are updated
+        # TODO (DONE): ensure somehow that variables are reserved for the inputs to the blocks
+        #              whose states are updated
+        #  executionLineForCurrentOrder.dependencySignalsSimulationInputs contains a list of input needed to update
+        #  the states.
         #
 
         signals_from_system_states = signalsToCache
@@ -343,15 +332,25 @@ def compile_single_system(system, reduce_uneeded_code = False):
         sUpCmd = CommandUpdateStates( blocksWhoseStatesToUpdate )
         commandsToExecuteForStateUpdate.append( sUpCmd )
 
-        #print("added command(s) to perform state update:")
-        #sUpCmd.printExecution()
-
         # get the dependendy singals of the current order
         # TODO important: remove the signals that are already computable from this list
         #dependencySignals = executionLineForCurrentOrder.dependencySignals
         dependencySignalsSimulationInputs = executionLineForCurrentOrder.dependencySignalsSimulationInputs
-        blocksToUpdateStates = executionLineForCurrentOrder.blocksToUpdateStates
-        dependencySignalsThroughStates = executionLineForCurrentOrder.dependencySignalsThroughStates
+        blocksToUpdateStates              = executionLineForCurrentOrder.blocksToUpdateStates
+        dependencySignalsThroughStates    = executionLineForCurrentOrder.dependencySignalsThroughStates
+
+
+        # TODO: handle special case in which a simulation input is requried for the state update of a block
+        #       and was before found to be required to calculate the outpus of sth. 
+        # instead of printing 'has already been calculated in a previous traversion' create an input to the update() function
+        #
+        # --- signals needed *indirectly* for s30 (through state update) --
+        # -> S osc_excitement
+        # -> S s22
+        # --- signals needed for s30 --
+        # -> S osc_excitement
+        # .  has already been calculated in a previous traversion
+
 
         # find out which singnals must be further computed to allow a state-update of the blocks
         dependencySignals__ = dependencySignalsThroughStates + dependencySignalsSimulationInputs
@@ -381,7 +380,7 @@ def compile_single_system(system, reduce_uneeded_code = False):
                                             generate_wrappper_functions = not reduce_uneeded_code )
 
     # code to reset add blocks in the simulation
-    commandsToExecuteForStateReset = CommandResetStates( blockList=blocksWhoseStatesToUpdate_All) # changed on 11.4.2020, before: sim.getBlocksArray()
+    commandsToExecuteForStateReset = CommandResetStates( blockList=blocksWhoseStatesToUpdate_All)
 
     # create an API-function resetStates()
     commandToResetStates = PutAPIFunction( nameAPI = 'resetStates', 
@@ -403,20 +402,17 @@ def compile_single_system(system, reduce_uneeded_code = False):
     allinputs.update( simulationInputSignalsToCalculateOutputs )
     allinputs = list(allinputs)
 
-
-
     # build the manifest for the compiled system
     manifest = SystemManifest( commandToExecute_system )
 
     compleResults = CompileResults( manifest, commandToExecute_system)
 
-    compleResults.inputSignals = allinputs
-    compleResults.simulationInputSignalsToUpdateStates = simulationInputSignalsToUpdateStates
+    compleResults.inputSignals                             = allinputs
+    compleResults.simulationInputSignalsToUpdateStates     = simulationInputSignalsToUpdateStates
     compleResults.simulationInputSignalsToCalculateOutputs = simulationInputSignalsToCalculateOutputs
-    compleResults.outputSignals = outputSignals
+    compleResults.outputSignals                            = outputSignals
 
     
-
     #
     return compleResults
 
