@@ -1,7 +1,7 @@
 from .libdyn import *
 from .Signal import *
 from .Block import *
-from .TraverseGraph import *
+from .TraverseSignalFlowGraph import *
 from . import  CodeGenHelper as cgh
 
 from typing import Dict, List
@@ -20,10 +20,17 @@ class ExecutionCommand(object):
         # the nesting level (by default 0)
         self.treeLevel_ = 0
 
+        # list of subcommands (filled in by derived classes)
+        self.executionCommands = []
+
         # the upper level execution command within the execution tree
         # None by default
         self.contextCommand = None
 
+        # object to define the tracing infrastructure (e.g. printf)
+        # in case of None, tracing is deactivated
+        self._tracing_infrastructure = None
+
     @property
     def treeLevel(self):
         return self.treeLevel_
@@ -33,15 +40,20 @@ class ExecutionCommand(object):
         self.contextCommand = context
         self.treeLevel_ = context.treeLevel + 1
 
-    def codeGen_init(self, language):
+    def set_tracing_infrastructure(self, tracing_infrastructure):
+        self._tracing_infrastructure = tracing_infrastructure
+
+        for cmd in self.executionCommands:
+            cmd.set_tracing_infrastructure( tracing_infrastructure )
+
+    def generate_code_init(self, language):
         # 
-        raise BaseException('codeGen_init unimplemented')
+        raise BaseException('generate_code_init unimplemented')
 
-    def codeGen_destruct(self, language):
-        raise BaseException('codeGen_destruct unimplemented')
-        pass
+    def generate_code_destruct(self, language):
+        raise BaseException('generate_code_destruct unimplemented')
 
-    def codeGen(self, language, flag):
+    def generate_code(self, language, flag):
 
         lines = ''
 
@@ -74,7 +86,7 @@ class CommandCalculateOutputs(ExecutionCommand):
         signalListStr = '['
 
         if self.targetSignals is not None:
-            signalListStr += cgh.signalListHelper_names_string(self.targetSignals)
+            signalListStr += cgh.signal_list_to_names_string(self.targetSignals)
 
         signalListStr += ']'
 
@@ -82,13 +94,13 @@ class CommandCalculateOutputs(ExecutionCommand):
 
         self.executionLine.printExecutionLine()
 
-    def codeGen_init(self, language):
+    def generate_code_init(self, language):
         pass
 
-    def codeGen_destruct(self, language):
+    def generate_code_destruct(self, language):
         pass
 
-    def codeGen(self, language, flag):
+    def generate_code(self, language, flag):
 
         lines = ''
 
@@ -113,7 +125,7 @@ class CommandCalculateOutputs(ExecutionCommand):
                             # notify the block prototype that the signal s will be a system output
                             # and, hence, no memory shall be allocated for s (because the memory is already
                             # available)
-                            s.getSourceBlock().getBlockPrototype().codeGen_setOutputReference('c++', s)
+                            s.getSourceBlock().getBlockPrototype().generate_code_setOutputReference('c++', s)
                             
                             
                             
@@ -131,14 +143,15 @@ class CommandCalculateOutputs(ExecutionCommand):
 
                             # print('create local variable for signal ' + s.name + ' / ' + s.toStr() )
 
-                            lines += cgh.defineVariableLine( s )
+                            if not s.is_referencing_memory:
+                                lines += cgh.defineVariableLine( s )
 
 
 
             if flag == 'code':
-                lines += '\n// calculating the block outputs in the following order ' + cgh.signalListHelper_names_string(self.executionLine.signalOrder ) + '\n'
-                lines += '// that depend on ' + cgh.signalListHelper_names_string(self.executionLine.dependencySignalsSimulationInputs) + '\n'
-                lines += '// dependencies that require a state update are ' + cgh.signalListHelper_names_string(self.executionLine.dependencySignalsThroughStates) + ' \n'
+                lines += '\n// calculating the block outputs in the following order ' + cgh.signal_list_to_names_string(self.executionLine.signalOrder ) + '\n'
+                lines += '// that depend on ' + cgh.signal_list_to_names_string(self.executionLine.dependencySignalsSimulationInputs) + '\n'
+                lines += '// dependencies that require a state update are ' + cgh.signal_list_to_names_string(self.executionLine.dependencySignalsThroughStates) + ' \n'
                 lines += '\n'
 
 
@@ -162,7 +175,7 @@ class CommandCalculateOutputs(ExecutionCommand):
                 # for each blocks that provides outputs that are needed to compute,
                 # generate the code to calculate these outputs.
                 for block in blocks_with_outputs_to_compute:
-                    lines += block.getBlockPrototype().codeGen_output_list('c++', blocks_with_outputs_to_compute[ block ] )
+                    lines += block.getBlockPrototype().generate_code_output_list('c++', blocks_with_outputs_to_compute[ block ] )
                     
         return lines
 
@@ -188,13 +201,13 @@ class CommandResetStates(ExecutionCommand):
 
         # self.executionLine.printExecutionLine()
 
-    def codeGen_init(self, language):
+    def generate_code_init(self, language):
         pass
 
-    def codeGen_destruct(self, language):
+    def generate_code_destruct(self, language):
         pass
 
-    def codeGen(self, language, flag):
+    def generate_code(self, language, flag):
 
         lines = ''
 
@@ -203,7 +216,7 @@ class CommandResetStates(ExecutionCommand):
             if flag == 'code':
                 lines += ''
                 for b in self.blockList:
-                    lines += b.getBlockPrototype().codeGen_reset('c++')
+                    lines += b.getBlockPrototype().generate_code_reset('c++')
 
         return lines
 
@@ -233,13 +246,13 @@ class CommandUpdateStates(ExecutionCommand):
 
         # self.executionLine.printExecutionLine()
 
-    def codeGen_init(self, language):
+    def generate_code_init(self, language):
         pass
 
-    def codeGen_destruct(self, language):
+    def generate_code_destruct(self, language):
         pass
 
-    def codeGen(self, language, flag):
+    def generate_code(self, language, flag):
 
         lines = ''
 
@@ -256,14 +269,14 @@ class CommandUpdateStates(ExecutionCommand):
                     # TODO: rename 'defStates' to 'variables'
                     #
                     
-                    lines += b.getBlockPrototype().codeGen_defStates('c++')
+                    lines += b.getBlockPrototype().generate_code_defStates('c++')
 
             if flag == 'code':
                 lines += '\n'
                 lines += ''
 
                 for b in self.blockList:
-                    lines += b.getBlockPrototype().codeGen_update('c++')
+                    lines += b.getBlockPrototype().generate_code_update('c++')
 
 
         return lines
@@ -296,13 +309,13 @@ class CommandCacheOutputs(ExecutionCommand):
         for s in self.signals:
             print("  - " + s.toStr() )
 
-    def codeGen_init(self, language):
+    def generate_code_init(self, language):
         pass
 
-    def codeGen_destruct(self, language):
+    def generate_code_destruct(self, language):
         pass
 
-    def codeGen(self, language, flag):
+    def generate_code(self, language, flag):
 
         lines = ''
 
@@ -311,24 +324,33 @@ class CommandCacheOutputs(ExecutionCommand):
 
             if flag == 'variables':
                 lines += ''
-                lines += "\n\n//\n// cached output values\n//\n"
+                lines += "\n\n//\n// cached output values\n//\n\n"
                 for s in self.signals:
 
                     cachevarName = s.name + "__" + s.getSourceBlock().getBlockPrototype().getUniqueVarnamePrefix()
 
-                    lines +=  '\n// cache for ' + s.name + '\n'
-                    lines +=  s.getDatatype().cppDataType + ' ' + cachevarName + "; // put NAN!" + '\n' 
+                    if not s.is_referencing_memory:
+                        # lines +=  '\n// cache for ' + s.name + '\n'
+                        lines +=  s.getDatatype().cpp_define_variable(cachevarName) + ";" + '\n' 
+
+                    else:
+                        comment = ' // cache for ' + s.name + ' (stores a pointer to a memory location)'
+                        lines +=  s.getDatatype().cpp_define_variable('(*' + cachevarName + ')') + ";" + comment + '\n' 
+
 
             if flag == 'code':
                 lines += '\n'
-                lines += '// saving the signals ' + cgh.signalListHelper_names_string(self.signals) + ' into the states \n'
-                lines += '\n'
+                lines += '// saving the signals ' + cgh.signal_list_to_names_string(self.signals) + ' into the states \n'
 
 
                 for s in self.signals:
-
                     cachevarName = s.name + "__" + s.getSourceBlock().getBlockPrototype().getUniqueVarnamePrefix()
-                    lines += cachevarName + ' = ' + s.name + ';\n'
+
+                    if not s.is_referencing_memory:
+                        lines += cachevarName + ' = ' + s.name + ';\n'
+                    else:
+                        # get the raw-pointer for the reference
+                        lines += cachevarName + ' = &(' + s.name + '); // just copy a pointer to the memory location\n'
 
         return lines
 
@@ -355,13 +377,13 @@ class CommandRestoreCache(ExecutionCommand):
         for s in self.signals:
             print("  - " + s.toStr() )
 
-    def codeGen_init(self, language):
+    def generate_code_init(self, language):
         pass
 
-    def codeGen_destruct(self, language):
+    def generate_code_destruct(self, language):
         pass
 
-    def codeGen(self, language, flag):
+    def generate_code(self, language, flag):
 
         lines = ''
 
@@ -369,13 +391,18 @@ class CommandRestoreCache(ExecutionCommand):
 
             if flag == 'code':
                 lines += '\n'
-                lines += '// restoring the signals ' + cgh.signalListHelper_names_string(self.signals) + ' from the states \n'
-                lines += '\n'
+                lines += '// restoring the signals ' + cgh.signal_list_to_names_string(self.signals) + ' from the states \n'
 
                 for s in self.signals:
 
                     cachevarName = s.name + "__" + s.getSourceBlock().getBlockPrototype().getUniqueVarnamePrefix()
-                    lines +=  s.getDatatype().cppDataType + ' ' + s.name + ' = ' + cachevarName + ";" + '\n' 
+
+                    if not s.is_referencing_memory:
+                        lines +=  s.getDatatype().cpp_define_variable( s.name, make_a_reference=True ) + ' = ' + cachevarName + ";" + '\n' 
+                    else:
+                        # set the reference to the memory the pointer 'cachevarName' is pointing to
+                        lines +=  s.getDatatype().cpp_define_variable( s.name, make_a_reference=True ) + ' = *' + cachevarName + ";" + ' // use a pointer to the memory location\n' 
+
 
                 lines += '\n'
 
@@ -428,15 +455,15 @@ class PutAPIFunction(ExecutionCommand):
 
         print(Style.BRIGHT + Fore.YELLOW + "}")
         
-    def codeGen_init(self, language):
+    def generate_code_init(self, language):
         for c in self.executionCommands:
-            c.codeGen_init(language)
+            c.generate_code_init(language)
 
-    def codeGen_destruct(self, language):
+    def generate_code_destruct(self, language):
         for c in self.executionCommands:
-            c.codeGen_destruct(language)
+            c.generate_code_destruct(language)
 
-    def codeGen(self, language, flag):
+    def generate_code(self, language, flag):
 
         lines = ''
 
@@ -444,7 +471,7 @@ class PutAPIFunction(ExecutionCommand):
 
             if flag == 'variables':
                 for c in self.executionCommands:
-                    lines += c.codeGen(language, 'variables')
+                    lines += c.generate_code(language, 'variables')
 
 
             if flag == 'code':
@@ -452,40 +479,41 @@ class PutAPIFunction(ExecutionCommand):
                 # ------ define the API-function ------
                 #
                 if len(self.outputSignals) > 0:
-                    lines += '// API-function ' + self._nameAPI + ' to compute: ' + cgh.signalListHelper_names_string( self.outputSignals )  #  ' '.join( cgh.signalListHelper_names( self.outputSignals ) )
+                    lines += '// API-function ' + self._nameAPI + ' to compute: ' + cgh.signal_list_to_names_string( self.outputSignals )
                 else:
-                    lines += '// API-function' + self._nameAPI
+                    lines += '// API-function ' + self._nameAPI
 
                 lines += '\n'
-                lines += 'void ' + self._nameAPI + '('
 
-                # put the parameter list e.g. double & y1, double & y2, u1, u2
-                elements = []
-                for s in self.outputSignals:
-                    elements.append( s.getDatatype().cppDataType + ' & '  + s.name )
-                    
-                elements.extend( cgh.signalListHelper_CppVarDefStr( self.inputSignals ) )
+                # innerLines will be put into the functions
+                function_code = ''
 
-                lines += ', '.join(elements)
-                lines +=  ') {\n'
+                # place tracing (begin)
+                if self._tracing_infrastructure is not None:
+                    function_code += cgh.create_printf(intro_string='ENTR: ' + self.contextCommand.API_name + '/' + self.nameAPI, 
+                                                    signals=self.inputSignals)
 
-                # innerLines will be indented
-                innerLines = ''
+                    function_code += '\n'
                 
                 # put the local variables
                 for c in self.executionCommands:
-                    innerLines += c.codeGen(language, 'localvar')
+                    function_code += c.generate_code(language, 'localvar')
                 
-                innerLines += '\n'
+                function_code += '\n'
 
                 # put the code
                 for c in self.executionCommands:
-                    innerLines += c.codeGen(language, 'code')
+                    function_code += c.generate_code(language, 'code')
 
-                lines += indent(innerLines, '  ')
+                # place tracing (end)
+                if self._tracing_infrastructure is not None:
+                    function_code += cgh.create_printf(intro_string='EXIT: ' + self.contextCommand.API_name + '/' + self.nameAPI, 
+                                                    signals=self.outputSignals)
 
-                # define the API-function (finish)
-                lines += '}\n\n'
+                    function_code += '\n'
+
+                # generate the function
+                lines += cgh.cpp_define_function(self._nameAPI, self.inputSignals, self.outputSignals, function_code )
 
                 #
                 # ------ end of 'define the API-function' ------
@@ -533,18 +561,18 @@ class PutAPIFunction(ExecutionCommand):
 
                     lines += '{\n'
 
-                    innerLines = ''
-                    innerLines += cgh.defineStructVar( 'Outputs_' + self._nameAPI, 'outputs'  ) + '\n'
+                    functionLines = ''
+                    functionLines += cgh.defineStructVar( 'Outputs_' + self._nameAPI, 'outputs'  ) + '\n'
 
-                    innerLines += '// call to wrapped function\n'
-                    innerLines += self._nameAPI + '(' + argumentsString + ');\n'
+                    functionLines += '// call to wrapped function\n'
+                    functionLines += self._nameAPI + '(' + argumentsString + ');\n'
 
 
-                    innerLines += '\n'
-                    innerLines += '// return the signals in a struct\n'
-                    innerLines += 'return outputs;\n'
+                    functionLines += '\n'
+                    functionLines += '// return the signals in a struct\n'
+                    functionLines += 'return outputs;\n'
 
-                    lines += indent(innerLines, '  ')
+                    lines += indent(functionLines, '  ')
                     
                     lines += '}\n\n'
                 else:
@@ -613,21 +641,21 @@ class PutSystem(ExecutionCommand):
 
         print(Style.BRIGHT + Fore.YELLOW + "}")
         
-    def codeGen_init(self, language):
+    def generate_code_init(self, language):
 
         for c in self.executionCommands:
-            c.codeGen_init(language)
+            c.generate_code_init(language)
 
         # call init codegen for each block in the simulation
         for block in self.system.blocks:
-            block.getBlockPrototype().codeGen_init(language)
+            block.getBlockPrototype().generate_code_init(language)
 
 
-    def codeGen_destruct(self, language):
+    def generate_code_destruct(self, language):
         for c in self.executionCommands:
-            c.codeGen_destruct(language)
+            c.generate_code_destruct(language)
 
-    def codeGen(self, language, flag):
+    def generate_code(self, language, flag):
 
         lines = ''
 
@@ -652,13 +680,13 @@ class PutSystem(ExecutionCommand):
                 innerLines = '// variables\n'
                 innerLines = 'public:\n'
                 for c in self.executionCommands:
-                    innerLines += c.codeGen(language, 'variables')
+                    innerLines += c.generate_code(language, 'variables')
                 
                 innerLines += '\n'
 
                 # put the code
                 for c in self.executionCommands:
-                    innerLines += c.codeGen(language, 'code')
+                    innerLines += c.generate_code(language, 'code')
 
                 lines += cgh.indent(innerLines)
 
@@ -700,16 +728,16 @@ class PutSystemAndSubsystems(ExecutionCommand):
 
         print(Style.BRIGHT + Fore.YELLOW + "}")
         
-    def codeGen_init(self, language):
+    def generate_code_init(self, language):
 
         for c in self.executionCommands:
-            c.codeGen_init(language)        
+            c.generate_code_init(language)        
 
-    def codeGen_destruct(self, language):
+    def generate_code_destruct(self, language):
         for c in self.executionCommands:
-            c.codeGen_destruct(language)            
+            c.generate_code_destruct(language)            
 
-    def codeGen(self, language, flag):
+    def generate_code(self, language, flag):
 
         lines = ''
 
@@ -728,13 +756,13 @@ class PutSystemAndSubsystems(ExecutionCommand):
                 innerLines = '// global variables\n'
 
                 for c in self.executionCommands:
-                    innerLines += c.codeGen(language, 'variables')
+                    innerLines += c.generate_code(language, 'variables')
 
                 innerLines += '\n'
 
                 # put the code
                 for c in self.executionCommands:
-                    innerLines += c.codeGen(language, 'code')
+                    innerLines += c.generate_code(language, 'code')
 
                 lines += cgh.indent(innerLines)
 
