@@ -1,5 +1,4 @@
 from contextlib import contextmanager
-#import contextvars
 from typing import Dict, List
 
 from .irpar import irparSet, irparElement, irparElement_container
@@ -9,14 +8,6 @@ from .Block import *
 from .DatatypePropagation import *
 
 
-# currentSimulation = 'none'
-
-# def detSimulationContext(sim):
-#     currentSimulation = sim
-
-# def showSimulationContext():
-#     print 
-
 
 # TODO: rename this to System
 class Simulation:
@@ -32,7 +23,6 @@ class Simulation:
         self._name = name
         self.BlocksArray = []
 
-
         # counter for system input signals
         # This determines the order of teh arguments of the generated c++ functions
         self.simulationInputSignalCounter = 0
@@ -105,8 +95,12 @@ class Simulation:
     def addBlock(self, blk : Block):
         self.BlocksArray.append(blk)
 
+    # TODO: remove this?
     def set_primary_outputs(self, outputSignals):
         self._output_signals = outputSignals
+
+    def append_primay_ouput(self, outputSignals):
+        self._output_signals.append(outputSignals)
     
     @property
     def primary_outputs(self):
@@ -298,11 +292,6 @@ class Simulation:
     def blocks(self):
         return self.BlocksArray
 
-    # TODO: remove
-    def getBlocksArray(self):
-        return self.BlocksArray
-
-
     def GetInputInterface(self):
         # Build an input-interface for the ORTD interpreter
         # inform of a "inlist" structure
@@ -324,15 +313,8 @@ class Simulation:
             block.verifyInputSignals(ignore_signals_with_datatype_inheritance)
 
     def propagate_datatypes(self):
-        #
-        #
-        #
-
         print("Propagating datatypes...")
 
-        # for block in self.BlocksArray:
-        #     block.verifyInputSignals(ignore_signals_with_datatype_inheritance=True)
-
         self.resolve_anonymous_signals(ignore_signals_with_datatype_inheritance=True)
 
         # find out the output datatypes
@@ -344,6 +326,12 @@ class Simulation:
         # for block in self.BlocksArray:
         #     block.verifyInputSignals(ignore_signals_with_datatype_inheritance=False)
 
+    @property
+    def signal_with_unresolved_datatypes(self):
+        return self.datatypePropagation.signalsWithUnderminedTypes
+
+
+    
 
         
 
